const db = require("../models");
const xml2js = require("xml2js");
const Model = db.model;
const Project = db.project;
const User = db.user;
const Op = db.Sequelize.Op;

function validatePerrmissions(projectId, userId, callback) {
    Project.findByPk(projectId,
        {
            include: [{
                model: User,
                as: "users",
                through: {
                    attributes: ["projectId", "userId"],
                },
            }]
        })
        .then(data => {
            if(data.users[0].id === userId || data.published) {
                callback(true)
            } else {
                callback(false)
            }
        })
        .catch(() => {
            callback(false)
        });
}

exports.createModel = (req, res) => {
    // Save Project to Database
    let projectId = req.query.projectId;

    if (!req.body.modelName || !req.body.modelDescription) {
        res.status(400).send({
            message: "Model name and description can not be empty!"
        });
        return;
    }

    // Create a Tutorial
    const model = {
        name: req.body.modelName,
        description: req.body.modelDescription,
        status: 'active',
        type: req.body.modelType ? req.body.modelType : 'default',
        projectId: projectId,
        data: req.body.modelData,
        updateBy: req.body.username,
    };

    validatePerrmissions(projectId, req.userId, (access) => {
        if(access) {
            Model.create(model)
                .then(result => {
                    res.status(200).send({ message: "Model was created successfully!", data: result});
                })
                .catch(err => {
                    res.status(500).send({ message: err.message });
                });
        } else {
            res.status(500).send({ message: "Unauthorized" });
        }
    });
};

exports.readModel = (req, res) => {
    const id = req.params.id;
    let projectId = req.query.projectId;

    validatePerrmissions(projectId, req.userId, (access) => {
        if(access) {
            if(!id) {
                res.status(500).send({
                    message: "Error retrieving Model with id=" + id
                });
            }
            Model.findByPk(id)
                .then(data => {
                    res.status(200).send(data);
                })
                .catch(err => {
                    res.status(500).send({
                        message: "Error retrieving Model with id=" + id
                    });
                });
        } else {
            res.status(500).send({ message: "Unauthorized" });
        }
    });
};

exports.updateModel = (req, res) => {
    const id = req.query.id;
    const saveModel = (req.query.saveModel) ? req.query.saveModel : false;
    let projectId = req.query.projectId;

    let model = {};

    if(saveModel) {
        model = {
            name: req.body.modelName,
            projectId: projectId,
            data: req.body.modelData,
            updateBy: req.body.username,
            description: req.body.modelDescription,
        };
    } else {
        model = {
            name: req.body.modelName,
            description: req.body.modelDescription,
            type: req.body.modelType,
            updateBy: req.body.username,
            projectId: projectId,
        };
    }

    validatePerrmissions(projectId, req.userId, (access) => {
        if(access) {
            Model.update(model, {
                where: { id: id }
            })
                .then((result) => {
                    res.status(200).send({
                        message: "Model was updated successfully.",
                        code: result
                    });
                })
                .catch(() => {
                    res.status(500).send({
                        message: "Error updating Model with id=" + id
                    });
                });
        } else {
            res.status(500).send({ message: "Unauthorized" });
        }
    });
};

exports.deleteModel = (req, res) => {

    const id = req.query.id;
    let projectId = req.query.projectId;

    validatePerrmissions(projectId, req.userId, (access) => {
        if(access) {
            Model.destroy({
                where: { id: id }
            })
                .then(num => {
                    if (num === 1) {
                        res.status(200).send({
                            message: "Model was deleted successfully!"
                        });
                    } else {
                        res.status(404).send({
                            message: `Cannot delete Metamdoel with id=${id}. Maybe Model was not found!`
                        });
                    }
                })
                .catch(err => {
                    res.status(500).send({
                        message: "Could not delete Project with id=" + id
                    });
                });
        } else {
            res.status(500).send({ message: "Unauthorized" });
        }
    });
};

exports.getAllModels = (req, res) => {

    let name = req.query.modelName;
    let projectId = req.query.projectId;

    let condition = name ? { name: { [Op.like]: `%${name}%` }, projectId: projectId } : { projectId: projectId };

    validatePerrmissions(projectId, req.userId, (access) => {
        if(access) {
            Model.findAll({ where: condition })
                .then(data => {
                    res.status(200).send(data);
                })
                .catch(err => {
                    res.status(500).send({
                        message:
                            err.message || "Some error occurred while retrieving projects."
                    });
                });
        } else {
            res.status(500).send({ message: "Unauthorized" });
        }
    });
};

exports.exportModel = (req, res) => {
    const id = req.query.id;
    const type = req.query.type;
    const reformat = req.query.reformat;

    if(type === 'json') {
        Model.findByPk(id)
            .then(data => {
                xml2js.parseString(data.data, (err, result) => {
                    if(err) {
                        res.status(500).send({
                            message: "Error retrieving Model with id=" + id + ", error: " + err
                        });
                    }

                    if(!reformat) {
                        return res.status(200).send(result);
                    }

                    let cells = result["mxGraphModel"]["root"][0]["mxCell"];
                    let newCells = [];

                    cells.forEach(cell => {
                        if(cell["mxGeometry"]) {
                            // cell["$"]["mxGeometry"] = cell["mxGeometry"];
                            delete cell["mxGeometry"];
                        }

                        newCells.push(cell["$"]);
                    });

                    let modelUuid = newCells[0]['id'];

                    // delete newCells[0];
                    // delete newCells[1]["parent"];

                    const idMapping = newCells.reduce((acc, el, i) => {
                        acc[el.id] = i;
                        return acc;
                    }, {});


                    let root = {};

                    newCells.forEach(el => {
                        delete el["style"];
                        delete el["connectable"];

                        if(el["value"] === "") {
                            delete el["value"];
                        }
                        // Handle the root element
                        if (!el.parent) {
                            root = el;
                            return;
                        }
                        // Use our mapping to locate the parent element in our data array
                        const parentEl = newCells[idMapping[el.parent]];
                        // Add our current el to its parent's `children` array
                        parentEl.children = [...(parentEl.children || []), el];
                    });

                    // root.children.forEach(child => {
                    //     delete child.parent;
                    // });

                    let responseData = {
                        modelId: data.id,
                        modelName: data.name,
                        modelDescription: data.description,
                        modelUuid: modelUuid,
                        modelData: root.children,
                        modelImage:'',
                    };
                    return res.status(200).send(responseData);
                });
            })
            .catch(err => {
                res.status(500).send({
                    message: "Error retrieving Model with id=" + id + ", error: " + err
                });
            });
    }
};


exports.importModel = (req, res) => {
    // Save Project to Database

    if (!req.body.name || !req.body.description || !req.body.type || !req.body.data) {
        res.status(400).send({
            message: "Content can not be empty!"
        });
        return;
    }

    const model = {
        name: req.body.name,
        description: req.body.description,
        status: req.body.status,
        type: req.body.type,
        projectId: req.body.projectId,
        data: req.body.data,
    };

    Model.create(model, )
        .then(() => {
            res.status(200).send({ message: "Model was created successfully!"});
        })
        .catch(err => {
            res.status(500).send({ message: err.message });
        });
};